/*
 * Draw_Buffer_Graphic.h
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#ifndef DRAW_BUFFER_GRAPHIC_H_
#define DRAW_BUFFER_GRAPHIC_H_

#include "Dev_Conf.h"

#include def_Header_HAL



//----------------------------------------------------------
// Graphicのサイズ
//----------------------------------------------------------
#define def_Widthwise_Graph    100

#define def_Lengthwise_Graph    100
//----------------------------------------------------------

//--------------------------------------------------------------
// Graphic Bufferのサイズ
//--------------------------------------------------------------
#define def_Size_Buffer_Graphic  \
    def_Widthwise_Graph * def_Lengthwise_Graph * def_RGB_byte
//--------------------------------------------------------------



//----------------------------------------------------------
// 描画Bufferに1dotの点を描画
//----------------------------------------------------------
// 引数;
// int16_t int16_X : 位置X(X座標)
// int16_t int16_Y : 位置Y(Y座標)

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawDot_Width_Height(int16_t int16_X, int16_t int16_Y,
                          int16_t int16_Type,
                          int16_t int16_AreaWidth,
                          int16_t int16_AreaHeight,
                          uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに1本の直線を描画
//----------------------------------------------------------
// int16_StartX : 開始位置X(X座標)
// int16_StartY : 開始位置Y(Y座標)

// int16_EndX : 終了位置X(X座標)
// int16_EndY : 終了位置Y(Y座標)

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawLine(int16_t int16_StartX, int16_t int16_StartY,
              int16_t int16_EndX, int16_t int16_EndY,
              int16_t int16_Type,
              int16_t int16_AreaWidth, int16_t int16_AreaHeight,
              uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに四角を描く(中は空)
//----------------------------------------------------------
// int16_StartX : 開始位置X(X座標)
// int16_StartY : 開始位置Y(Y座標)

// int16_EndX : 終了位置X(X座標)
// int16_EndY : 終了位置Y(Y座標)

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawRect(int16_t int16_StartX, int16_t int16_StartY,
              int16_t int16_EndX, int16_t int16_EndY,
              int16_t int16_Type,
              int16_t int16_AreaWidth, int16_t int16_AreaHeight,
              uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに四角を描く(塗りつぶす)
//----------------------------------------------------------
// int16_StartX : 開始位置X(X座標)
// int16_StartY : 開始位置Y(Y座標)

// int16_EndX : 終了位置X(X座標)
// int16_EndY : 終了位置Y(Y座標)

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawRectP(int16_t int16_StartX, int16_t int16_StartY,
               int16_t int16_EndX, int16_t int16_EndY,
               int16_t int16_Type,
               int16_t int16_AreaWidth, int16_t int16_AreaHeight,
               uint8_t *puint8_aDrawBuffer);


//----------------------------------------------------------
// 描画Bufferに円を描く(中は空)
//----------------------------------------------------------
// int16_CenterX : 円の中心点X(X座標)
// int16_CenterY : 円の中心点Y(Y座標)

// int16_Radius : 円の半径

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawCircle(int16_t int16_CenterX, int16_t int16_CenterY,
                int16_t int16_Radius,
                int16_t int16_Type,
                int16_t int16_AreaWidth, int16_t int16_AreaHeight,
                uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに円を描く(塗りつぶす)
//----------------------------------------------------------
// int16_CenterX : 円の中心点X(X座標)
// int16_CenterY : 円の中心点Y(Y座標)

// int16_Radius : 円の半径

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void DrawCircleP(int16_t int16_CenterX, int16_t int16_CenterY,
                 int16_t int16_Radius,
                 int16_t int16_Type,
                 int16_t int16_AreaWidth, int16_t int16_AreaHeight,
                 uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに円弧を描く(中は空) :
//----------------------------------------------------------
// int16_CenterX : 円の中心点X(X座標)
// int16_CenterY : 円の中心点Y(Y座標)

// int16_Radius : 円の半径

// int16_t int16_Start : 描画開始位置   0-359
// int16_t int16_End : 描画終了位置   int16_Start-359

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Draw_Arc(int16_t int16_CenterX, int16_t int16_CenterY,
              int16_t int16_Radius,
              int16_t int16_Start, int16_t int16_End,
              int16_t int16_Type,
              int16_t int16_AreaWidth, int16_t int16_AreaHeight,
              uint8_t *puint8_aDrawBuffer);



//----------------------------------------------------------
// 描画Bufferに円弧を描く(塗りつぶす)
//----------------------------------------------------------
// int16_CenterX : 円の中心点X(X座標)
// int16_CenterY : 円の中心点Y(Y座標)

// int16_Radius : 円の半径

// int16_t int16_Start : 描画開始位置   0-359
// int16_t int16_End : 描画終了位置   int16_Start-359

// int16_Type :
//    0 : 描画dot OFF
//    1 : 描画dot ON

// int16_t int16_AreaWidth : 描画領域の幅
// int16_t int16_AreaHeight : 描画領域の高さ

// uint8_t *puint8_aDrawBuffer : 描画Bufferのポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Draw_Arc_P(int16_t int16_CenterX, int16_t int16_CenterY,
                int16_t int16_Radius,
                int16_t int16_Start, int16_t int16_End,
                int16_t int16_Type,
                int16_t int16_AreaWidth, int16_t int16_AreaHeight,
                uint8_t *puint8_aDrawBuffer);



// Graphic 描画Buffer
extern uint8_t GLB_uint8_aBuffer_Graphic[def_Size_Buffer_Graphic];



#endif /* DRAW_BUFFER_GRAPHIC_H_ */
