/*
 * ILI9341_SPI_Draw.h
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#ifndef ILI9341_SPI_DRAW_H_
#define ILI9341_SPI_DRAW_H_

#include "Dev_Conf.h"

#include def_Header_HAL



//----------------------------------------------------------
// font選択番号の定義
//----------------------------------------------------------
// ASCII
//----------------------------------------------------------
#define def_Font_ASCII_8x8             0  //  8 x  8 font
#define def_Font_ASCII_16x16           1  // 16 x 16 font
#define def_Font_ASCII_Expand_32x32    2  // 32 x 32 font
#define def_Font_ASCII_Expand_48x48    3  // 48 x 48 font
//----------------------------------------------------------
// Unicode
//----------------------------------------------------------
#define def_Font_Unicode_16x16         4  // 16 x 16 font
#define def_Font_Unicode_Expand_32x32  5  // 32 x 32 font
#define def_Font_Unicode_Expand_48x48  6  // 48 x 48 font
//----------------------------------------------------------



//--------------------------------------
// 縦長/横長 選択
//--------------------------------------
#define def_LengthwiseL 0  // 縦長 左
#define def_LengthwiseR 1  // 縦長 右
#define def_WidthwiseL  2  // 横長 左
#define def_WidthwiseR  3  // 横長 右
//--------------------------------------



//------------------------------------------------------
// LCD表示方向の選択 :
//      モニタ画面などで一定方向表示の場合、使用する。
//------------------------------------------------------
//------------------------------
#if def_Dir_Disp_LCD == 0
//------------------------------
#define def_Dir_Disp_LCD_Sel  def_LengthwiseL
//------------------------------
#elif def_Dir_Disp_LCD == 1
//------------------------------
#define def_Dir_Disp_LCD_Sel  def_LengthwiseR
//------------------------------
#elif def_Dir_Disp_LCD == 2
//------------------------------
#define def_Dir_Disp_LCD_Sel  def_WidthwiseL
//------------------------------
#elif def_Dir_Disp_LCD == 3
//------------------------------
#define def_Dir_Disp_LCD_Sel  def_WidthwiseR
//------------------------------
#else
//------------------------------
#define def_Dir_Disp_LCD_Sel  def_WidthwiseR
//------------------------------
#endif
//------------------------------------------------------





//----------------------------------
// LCDの表示サイズ : 320 x 240
//----------------------------------
#define def_DotWidthwise_LCD   320
#define def_DotLengthwise_LCD  240


//------------------------------------------------------
// 各Fontの表示文字数とサイズ
//------------------------------------------------------
//--------------------------------------------------
// 8 x 8 Font
//--------------------------------------------------
#define def_NumColumn_Widthwise_Font8x8  40
#define def_NumRow_Lengthwise_Font8x8    30

#define def_DotWidthwise_Font8x8   8
#define def_DotLengthwise_Font8x8  8


//--------------------------------------------------
// 16 x 16 Font
//--------------------------------------------------
#define def_NumColumn_Widthwise_Font16x16  20
#define def_NumRow_Lengthwise_Font16x16    15

#define def_DotWidthwise_Font16x16   16
#define def_DotLengthwise_Font16x16  16


//--------------------------------------------------
// 32 x 32 Font
//--------------------------------------------------
#define def_NumColumn_Widthwise_Font32x32  10
#define def_NumRow_Lengthwise_Font32x32     7

#define def_DotWidthwise_Font32x32   32
#define def_DotLengthwise_Font32x32  32
//--------------------------------------------------


//--------------------------------------------------
// 48 x 48 Font
//--------------------------------------------------
#define def_NumColumn_Widthwise_Font48x48  6
#define def_NumRow_Lengthwise_Font48x48    5

#define def_DotWidthwise_Font48x48   48
#define def_DotLengthwise_Font48x48  48
//--------------------------------------------------
//------------------------------------------------------



//------------------------------------------------------
// DrawBuffer_LCD のサイズ
//------------------------------------------------------
#define def_Size_DrawBuffer_LCD  \
            (def_NumColumn_Widthwise_Font8x8 \
                            * def_DotLengthwise_LCD)



//------------------------------------------------------
// LCDに送信する1Line分のデータ数
//------------------------------------------------------
#define def_Number_Send_1Line_LCD  \
        (def_DotWidthwise_LCD * def_RGB_byte + 256)
//------------------------------------------------------



//--------------------------------------------------
// LCD描画Frame設定のための構造体
//--------------------------------------------------
struct sFrame_LCD{
  uint16_t uint16_SC;  // Start Column
  uint16_t uint16_EC;  // End Column
  uint16_t uint16_SP;  // Start Page
  uint16_t uint16_EP;  // End Page
};
//--------------------------------------------------



//----------------------------------------------------------
// ILI9341のRegister設定
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// int16_t int16_NSS_OnOff :
//                      設定データ送信終了時NSS ON/OFF選択
//    0 : 送信後 NSSをOFFにする。
//    1 : 送信後 NSSをONのままにする。

// uint8_t uint8_Address : Address of Register
// uint16_t uint16_DataLength : 書き込みデータ数
// uint8_t *puint8_aData :
//          書き込みデータが格納されているBufferのポインタ


// 戻り値;
//   -1 : 処理中
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t ILI9341_SPI_WriteRegisteer(uint8_t *puint8_JobNum,
                                    int16_t int16_NSS_OnOff,
                                    uint8_t uint8_Address,
                                    uint16_t uint16_DataLength,
                                    uint8_t *puint8_aData);



//----------------------------------------------------------
// 表示Frame設定 : ILI9341
//----------------------------------------------------------
// 表示する向きと表示位置を設定する。
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// uint16_t uint16_Oblong : 縦長/横長  選択
//    0 : Lengthwise : 縦長 左
//    1 : Lengthwise : 縦長 右
//    2 : WidthwiseL : 横長 左
//    3 : WidthwiseR : 横長 右

// struct sFrame_LCD *pstructFrame_LCD :
//                      表示パラメータ構造体のポインタ


// 戻り値;
//   -1 : 処理中
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t ILI9341_SPI_SetFrame(uint8_t *puint8_JobNum,
                             uint16_t uint16_Oblong,
                        struct sFrame_LCD *pstructFrame_LCD);



//----------------------------------------------------------
// ILI9341のレジスタ設定と表示Data送信
//----------------------------------------------------------
// 引数;
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// int16_t int16_NSS_OnOff :
//                      設定データ送信終了時NSS ON/OFF選択
//    0 : 送信後 NSSをOFFにする。
//    1 : 送信後 NSSをONのままにする。

// uint16_t uint16_Length : 送信データ数
// uint8_t *puint8_aData :
//              送信データが格納されているBufferのポインタ


// 戻り値;
//   -1 : 処理中
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t ILI9341_SPI_SendData(uint8_t *puint8_JobNum,
                             int16_t int16_NSS_OnOff,
                             uint16_t uint16_Length,
                             uint8_t *puint8_aData);



//----------------------------------------------------------
// LCD表示用送信データ作成(カラーコード指定)  : RGB 2byte
//----------------------------------------------------------
// 24bitのカラーコードから 2byte のRGBデータを作成する。
//----------------------------------------------------------
// 引数:
// uint32_t uint32_ColorCode : カラーコード

// uint16_t uint16_Width : 表示領域の幅
// uint16_t uint16_Position : DrawDataの位置
// uint8_t *puint8_aDrawData :
//                      Drawデータが格納されたBufferのポインタ
// uint8_t *puint8_aSendData :
//                      送信データを格納するBufferのポインタ


// 戻り値;
//    なし
//--------------------------------------------------------------
void MakeSendData_1Line_ColorCode_2byte(
                                    uint32_t uint32_ColorCode,
                                    uint16_t uint16_Width,
                                    uint16_t uint16_Position,
                                    uint8_t *puint8_aDrawData,
                                    uint8_t *puint8_aSendData);



extern int8_t GLB_int8_Semaphore_LCD;
                                // LCD表示のためのSemaphore


//----------------------------------------------------------
// LCD画面 Font 48 x 48 文字 1行分のBufferを含む
// 20 dot x 240 line : 40byte x 240 line 分のBuffer
//----------------------------------------------------------
extern uint8_t GLB_uint8_aDrawBuffer_LCD[
                        (uint16_t)def_Size_DrawBuffer_LCD];

extern uint8_t GLB_uint8_aBuffer_1Line_LCD[
                        (uint16_t)def_Number_Send_1Line_LCD];
                            // LCDに送信する1Line分のBuffer

extern uint8_t GLB_uint8_aWork_1Line_LCD[
                    (uint16_t)def_Number_Send_1Line_LCD * 2];
                            // LCDに送信する1Line分のWork



#endif /* ILI9341_SPI_DRAW_H_ */
