/*
 * Dev_Conf.h
 *
 *  Created on: 2025/02/26
 *      Author: egaono-electronics
 */

#ifndef DEV_CONF_H_
#define DEV_CONF_H_


//------------------------------------------
// 使用するCPUの選択
//------------------------------------------
// STM32F446RE
//------------------------------------------
#define def_USE_F446RE
//------------------------------------------

//------------------------------------------
// HALヘッダの定義
//------------------------------------------
#define def_Header_HAL  "stm32f4xx_hal.h"
//------------------------------------------

//----------------------------------------------------------
// Peripheralヘッダの定義
//----------------------------------------------------------
#define def_H_Periph_FLASH  "H_Common_STM32F4_FLASH.h"
#define def_H_Periph_PWM    "H_Common_STM32F4_PWM_1CH.h"
#define def_H_Periph_SPI    "H_STM32F4_SPI_1_2.h"
//----------------------------------------------------------


//----------------------------------------------------------
// CPU動作周波数の定義
//----------------------------------------------------------
#define def_SystemClock    180000000  // 最高速
//----------------------------------------------------------


//--------------------------------------------------------------
// ST LEDの定義
//--------------------------------------------------------------
//----------------------------------------------------------
// ST LEDのポートとピンの定義 : Base-F446RE基板のST1
//----------------------------------------------------------
#define  def_GPIOx_LED_ST       GPIOA
#define  def_GPIO_PIN_x_LED_ST  GPIO_PIN_7

#define def_GPIOx_CLK_ENABLE_LED_ST()  \
	                        __HAL_RCC_GPIOA_CLK_ENABLE()
//----------------------------------------------------------
//--------------------------------------------------------------


//--------------------------------------------------------------
// User Switchの定義
//--------------------------------------------------------------
//----------------------------------------------------------
// User Switchのポートとピンの定義 :
//                          Base-F446RE基板のSW1 : USER
//----------------------------------------------------------
#define def_GPIOx_USW       GPIOC
#define def_GPIO_PIN_x_USW  GPIO_PIN_13

#define def_GPIOx_CLK_ENABLE_USW()  \
	                        __HAL_RCC_GPIOC_CLK_ENABLE()
//----------------------------------------------------------


//--------------------------------------------------
// User Switch検出の定義
//--------------------------------------------------
#define def_H_Detect_USW  "Detects_Pressed_USW.h"

#define def_M_Initialize_USW  Initialize_USW
#define def_M_Detect_USW  Detects_Pressed_USW

#define def_USW_OFF  GPIO_PIN_SET
#define def_USW_ON   GPIO_PIN_RESET
//--------------------------------------------------
//--------------------------------------------------------------



//----------------------------------------------------------
// LCD制御の定義
//----------------------------------------------------------


//----------------------------------------------
// RGBのbyte数の選択
//----------------------------------------------
//----------------------------------------------
#define def_LCD_Lengthwise  240
#define def_LCD_Widthwise   320
#define def_RGB_byte          2  // RGB 2byte


//----------------------------------------------------------
// LCD表示の向きを指定する。
//----------------------------------------------------------
//#define def_Dir_Disp_LCD  0  // def_LengthwiseL : 縦長 左
//#define def_Dir_Disp_LCD  1  // def_LengthwiseR : 縦長 右
#define def_Dir_Disp_LCD  2  // def_WidthwiseL : 横長 左
//#define def_Dir_Disp_LCD  3  // def_WidthwiseR : 横長 右
//----------------------------------------------------------


//----------------------------------------------------------
//--------------------------------------------------
// LCD : LEDピン : BackLight制御ピンの定義
//--------------------------------------------------
#define def_GPIOx_LCD_LED         GPIOA
#define def_GPIO_PIN_x_LCD_LED    GPIO_PIN_9


//--------------------------------------------------
// LCD : DCピンの定義
//--------------------------------------------------
#define def_GPIOx_LCD_DC         GPIOC
#define def_GPIO_PIN_x_LCD_DC    GPIO_PIN_2


//--------------------------------------------------
// LCD : RESETピンの定義
//--------------------------------------------------
#define def_GPIOx_LCD_Reset         GPIOA
#define def_GPIO_PIN_x_LCD_Reset    GPIO_PIN_10
//----------------------------------------------------------


//--------------------------------------------------
// LCD : CSピンの定義
//--------------------------------------------------
#define def_GPIOx_LCD_SPI_NSS         GPIOB
                            // def_GPIOx_SPI1_NSS
#define def_GPIO_PIN_x_LCD_SPI_NSS    GPIO_PIN_2
                            // def_GPIO_PIN_x_SPI1_NSS
//----------------------------------------------------------



//-------------------------------------
// LCD Touch Panel : T_IRQピンの定義
//-------------------------------------
//------------------------------------------------------------------
#define def_GPIOx_LCD_T_IRQ       GPIOC
#define def_GPIO_PIN_x_LCD_T_IRQ  GPIO_PIN_3
//------------------------------------------------------------------



//------------------------------------------------------------------
// タッチパネル制御に使用するSPIの定義
//------------------------------------------------------------------
#define def_SPIx_TP  SPI2
#define def_Handle_SPIx_TP  Handle_SPI2

#define def_uint8_SPIx_TxBuffer  \
	            GLB_structParam_Handle_SPI2.puint8_aSPI_TxBuffer
#define def_uint8_SPIx_RxBuffer  \
	            GLB_structParam_Handle_SPI2.puint8_aSPI_RxBuffer

#define def_int16_vSPIx_TxNumber  \
	            GLB_structParam_Handle_SPI2.int16_vSPI_TxNumber

#define def_SPIx_Master_TxRxByte  SPIx_Master_TxRxByte

#define def_structParam_Handle_SPIx  GLB_structParam_Handle_SPI2
//------------------------------------------------------------------


//--------------------------------------------------
// タッチパネルのY位置補正をする場合、定義する。
//--------------------------------------------------
#define def_Correct_Pos_TP_Y
//--------------------------------------------------


//--------------------------------------------------
// タッチパネルスイッチ反転表示時間 : mSec
//--------------------------------------------------
#define def_Time_Print_TP_SW_Reversal  150
//--------------------------------------------------



//----------------------------------------------------------
// SPI1 の定義
//----------------------------------------------------------

//------------------------------------------------------
// SPI1 Peripheral定義
//------------------------------------------------------
#define def_GPIOx_SPI1    GPIOB

#define def_GPIO_PIN_x_SPI1_SCK   GPIO_PIN_3
#define def_GPIO_PIN_x_SPI1_MISO  GPIO_PIN_4
#define def_GPIO_PIN_x_SPI1_MOSI  GPIO_PIN_5

#define def_GPIOx_SPI1_NSS        GPIOB
#define def_GPIO_PIN_x_SPI1_NSS   GPIO_PIN_2
//------------------------------------------------------


//------------------------------------------------------
// SPI1 Bufferサイズ : LCD 1Line分の送信データ数
//------------------------------------------------------
#define def_SPI1_TxBufferSize  \
                def_LCD_Widthwise * def_RGB_byte + 8
                        // LCDの1Line分の送信データ数 :
                    // 320dot x RGB + Command + Address
#define def_SPI1_RxBufferSize  def_SPI1_TxBufferSize
//------------------------------------------------------
//----------------------------------------------------------



//----------------------------------------------------------
// SPI2 の定義
//----------------------------------------------------------

//------------------------------------------------------
// SPI2 Peripheral定義
//------------------------------------------------------
#define def_GPIOx_SPI2    GPIOB

#define def_GPIO_PIN_x_SPI2_SCK   GPIO_PIN_13
#define def_GPIO_PIN_x_SPI2_MISO  GPIO_PIN_14
#define def_GPIO_PIN_x_SPI2_MOSI  GPIO_PIN_15

#define def_GPIOx_SPI2_NSS        GPIOB
#define def_GPIO_PIN_x_SPI2_NSS   GPIO_PIN_12
//------------------------------------------------------


//--------------------------------------
// SPI2 Bufferサイズ
//--------------------------------------
#define def_SPI2_TxBufferSize    1024
#define def_SPI2_RxBufferSize    1024
//--------------------------------------
//----------------------------------------------------------



//----------------------------------------------------------
// SPI_BAUDRATEPRESCALER の定義
//----------------------------------------------------------
// SPI_SCK が 10MHz になるように設定する。
//----------------------------------------------------------
#define def_SPI_BAUDRATEPRESCALER_x  SPI_BAUDRATEPRESCALER_8
//----------------------------------------------------------



//------------------------------------------------------------------
// Beep制御の定義
//------------------------------------------------------------------
//------------------------------------------------------
// BeepにPWMを使用する場合、定義する。
//------------------------------------------------------
#define def_USE_Beep_PWM
//------------------------------------------------------



//--------------------------------------------------------------
// PWMに使用するピンの定義 : TIM3 CH1 : Beep
//--------------------------------------------------------------
#define def_GPIOx_CLK_ENABLE_PWM_TIM3_CH1()  \
                                __HAL_RCC_GPIOA_CLK_ENABLE()

#define  def_GPIOx_PWM_TIM3_CH1    GPIOA

#define  def_GPIO_PIN_PWM_TIM3_CH1  GPIO_PIN_6
//--------------------------------------------------------------



//--------------------------------------------------------------
// PWMに使用するTIMの定義 : Beep
//--------------------------------------------------------------
#define def_TIMx_Beep_PWM    TIM3

#define def_GPIO_AFx_TIMx_Beep_PWM  GPIO_AF2_TIM3

#define def_TIMx_CLK_ENABLE_Beep_PWM()  \
                            __HAL_RCC_TIM3_CLK_ENABLE()

#define def_TIM_CHANNEL_x_Beep_PWM  TIM_CHANNEL_1
//--------------------------------------------------------------


//--------------------------------------------------------------
// PWMに使用するGPIOポートとピンの定義 : Beep
//--------------------------------------------------------------
// CH1
#define def_GPIOx_CLK_ENABLE_Beep_PWM_CH()  \
                                __HAL_RCC_GPIOA_CLK_ENABLE()


#define  def_GPIOx_Beep_PWM_CH    GPIOA

#define  def_GPIO_PIN_x_Beep_PWM_CH    GPIO_PIN_6
//--------------------------------------------------------------


//--------------------------------------------------
// Beep ON時間 : mSec
//--------------------------------------------------
#define def_Time_Beep    25
//--------------------------------------------------


//--------------------------------------------------
// Beepを繰り返し鳴らす場合の間隔
//--------------------------------------------------
#define def_Interval_Beep_Repeat    75
//--------------------------------------------------
//------------------------------------------------------------------


//--------------------------------------------------------------
// PWM周期設定のTIMxクロックの分周比
//--------------------------------------------------------------
#define def_Div_Beep_PWM  1
//--------------------------------------------------------------



//--------------------------------------------------------------
// 使用Fontの定義
//--------------------------------------------------------------
#define def_Font_ASCII_8x8_Array  FONT_uint8_aASCII_8x8_A
#define def_Font_ASCII_16x16_Array  FONT_uint8_aASCII_16x16_A
#define def_Font_HEX_16x16_Array  FONT_uint8_aHEX_16x16_A
//--------------------------------------------------------------


//------------------------------------------------------
// 使用Fontのヘッダ定義
//------------------------------------------------------
#define def_H_Font_ASCII_8x8  "Font_ASCII_8x8_A.h"
#define def_H_Font_ASCII_16x16  "Font_ASCII_16x16_A.h"
//------------------------------------------------------


//--------------------------------------------------
// FLASH制御関数の定義
//--------------------------------------------------
#define def_EraseFLASH  EraseFLASH
#define def_SetDataAndWriteFLASH  SetDataAndWriteFLASH

// Flash Memoryのサイズが 1Mバイトの場合、定義する。
//#define def_USE_FLASH_1M
//--------------------------------------------------



//----------------------------------------------------------
// Flash Memory上の各パラメータ格納アドレス
//----------------------------------------------------------

//--------------------------------------------------
// シリアルテスタパラメータ格納開始アドレス
//--------------------------------------------------
#define def_Address_Param_SCT    0x08008000
//--------------------------------------------------


//--------------------------------------------------
// タッチパネル調整済み文字列エリアの先頭アドレス
//--------------------------------------------------
#define def_Address_Adjust_TP  0x080080E0
//--------------------------------------------------

//----------------------------------------------------------


//----------------------------------------------------------
// パラメータ セーブ/ロード モジュールヘッダの定義
//----------------------------------------------------------
#define def_H_Tester_Save_Load  "MSP2807_Test_TP_Save_Load.h"


#endif /* DEV_CONF_H_ */
