/*
 * MSP2807_Startup_Screen_Test_TP.c
 *
 *  Created on: 2025/02/25
 *      Author: egaono-electronics
 */

#include "MSP2807_Startup_Screen_Test_TP.h"

#include "Color_Code.h"

#include "ILI9341_SPI_Draw.h"
#include "ILI9341_Display_Clear.h"
#include "ILI9341_Display_ASCII.h"


//--------------------------------------------------------------
// タッチパネル試験 MSP2807起動画面表示 : Touch Panel
//--------------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//--------------------------------------------------------------
int16_t MSP2807_Startup_Screen_Test_TP(uint8_t *puint8_JobNum)
{
  static uint8_t STC_uint8_JobNum_LCD;

  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    STC_uint8_JobNum_LCD = 0;  // LCD表示処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------------------------------
    // 画面表示をクリアする。
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK
    //    1 : NG
    //--------------------------------------------------------------
    int16_Return = Clear_Display(&STC_uint8_JobNum_LCD);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return != 0){
      *puint8_JobNum = 0;  // 処理番号初期化
      break;  // NG終了
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //--------------------------------------------------------------
    // font 32 x 32 : ASCII : "Touch"
    //--------------------------------------------------------------

    //--------------------------------------------------
    // 横長表示 左
    //--------------------------------------------------
    //--------------------------------------------------------------
    // LCD文字表示 : ASCII : Font選択 : ILI9341
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // int16_t int16_Check_Semaphore : セマフォチェック有無
    //    0 : チェックなし
    //    1 : チェックあり

    // uint16_t uint16_Font : font指定
    //    0 : 8 x 8 font : ASCII
    //    1 : 16 x 16 font : ASCII
    //    2 : 32 x 32 font : ASCII
    //    3 : 48 x 48 font : ASCII

    // uint8_t uint8_Oblong : 縦長/横長
    //    0 : Lengthwise : 縦長 左
    //    1 : Lengthwise : 縦長 右
    //    2 : WidthwiseL : 横長 左
    //    3 : WidthwiseR : 横長 右

    // uint8_t uint8_Type : 表示選択
    //      0 : 通常表示
    //      1 : 反転表示
    //      2 : 通常表示の上書き
    //      3 : 反転表示の上書き

    // uint32_t uint32_ColorCode : カラーコード

    // uint16_t uint16_StartX : 表示開始位置X
    // uint16_t uint16_StartY : 表示開始位置Y

    // uint16_t uint16_Length : 表示文字数
    // uint8_t *puint8_Data :
    //          表示文字コードが格納されているBufferのポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = ILI9341_SPI_Display_ASCII(&STC_uint8_JobNum_LCD,
                                1,
                                (uint16_t)def_Font_ASCII_Expand_32x32,
                                def_WidthwiseL, 0,
                                (uint32_t)def_Code_Darkmagenta,
                                64, 56,
                                5, (uint8_t *)"Touch");
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    //--------------------------------------------------------------
    // font 32 x 32 : ASCII : "Panel"
    //--------------------------------------------------------------

    //--------------------------------------------------
    // 横長表示 左
    //--------------------------------------------------
    //--------------------------------------------------------------
    // LCD文字表示 : ASCII : Font選択 : ILI9341
    //--------------------------------------------------------------
    int16_Return = ILI9341_SPI_Display_ASCII(&STC_uint8_JobNum_LCD,
                                1,
                                (uint16_t)def_Font_ASCII_Expand_32x32,
                                def_WidthwiseL, 0,
                                (uint32_t)def_Code_Hotpink,
                                96, 88,
                                5, (uint8_t *)"Panel");
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    
    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 4:
    //--------------------------------------------------------------
    // font 32 x 32 : ASCII : "Operation"
    //--------------------------------------------------------------

    //--------------------------------------------------
    // 横長表示 左
    //--------------------------------------------------
    //--------------------------------------------------------------
    // LCD文字表示 : ASCII : Font選択 : ILI9341
    //--------------------------------------------------------------
    int16_Return = ILI9341_SPI_Display_ASCII(&STC_uint8_JobNum_LCD,
                                1,
                                (uint16_t)def_Font_ASCII_Expand_32x32,
                                def_WidthwiseL, 0,
                                (uint32_t)def_Code_Cyan,
                                16, 120,
                                9, (uint8_t *)"Operation");
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 0;    // OK終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}
